package betterwithmods.common.blocks.mechanical.tile;

import betterwithmods.api.capabilities.CapabilityMechanicalPower;
import betterwithmods.api.tile.IMechanicalPower;
import betterwithmods.common.blocks.mechanical.BlockCookingPot;
import betterwithmods.common.blocks.tile.TileEntityVisibleInventory;
import betterwithmods.common.registry.bulk.manager.CraftingManagerBulk;
import betterwithmods.common.registry.heat.BWMHeatRegistry;
import betterwithmods.common.registry.heat.BWMHeatSource;
import betterwithmods.util.DirUtils;
import betterwithmods.util.InvUtils;
import betterwithmods.util.MechanicalUtil;
import net.minecraft.block.Block;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.item.EntityItem;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Items;
import net.minecraft.init.SoundEvents;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.EntitySelectors;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.NonNullList;
import net.minecraft.util.SoundCategory;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.Vec3i;
import net.minecraft.world.World;
import net.minecraftforge.common.capabilities.Capability;
import net.minecraftforge.oredict.OreDictionary;

import java.util.List;


public abstract class TileEntityCookingPot extends TileEntityVisibleInventory implements IMechanicalPower {
    public int cookCounter;
    public int stokedCooldownCounter;
    public int scaledCookCounter;
    public boolean containsValidIngredients;
    public int fireIntensity;
    public EnumFacing facing;
    protected CraftingManagerBulk unstoked, stoked;
    private boolean forceValidation;

    public TileEntityCookingPot(CraftingManagerBulk unstoked, CraftingManagerBulk stoked) {
        this.unstoked = unstoked;
        this.stoked = stoked;
        this.cookCounter = 0;
        this.containsValidIngredients = false;
        this.forceValidation = true;
        this.scaledCookCounter = 0;
        this.fireIntensity = -1;
        this.occupiedSlots = 0;
        this.facing = EnumFacing.UP;
    }

    @Override
    public boolean hasFastRenderer() {
        return true;
    }

    @Override
    public boolean hasCapability(Capability<?> capability, EnumFacing facing) {
        return capability == CapabilityMechanicalPower.MECHANICAL_POWER || super.hasCapability(capability, facing);
    }

    @Override
    public <T> T getCapability(Capability<T> capability, EnumFacing facing) {
        if (capability == CapabilityMechanicalPower.MECHANICAL_POWER) {
            return CapabilityMechanicalPower.MECHANICAL_POWER.cast(this);
        }
        return super.getCapability(capability, facing);
    }

    private boolean isInputtingPower(EnumFacing facing) {
        return this.getMechanicalInput(facing) > 0;
    }

    private EnumFacing getPoweredSide() {
        for (EnumFacing facing : EnumFacing.field_176754_o) {
            if (isInputtingPower(facing))
                return facing;
        }
        return null;
    }

    private boolean isPowered() {
        for (EnumFacing facing : EnumFacing.field_176754_o) {
            if (isInputtingPower(facing))
                return true;
        }
        return false;
    }

    protected boolean isStoked() {
        return fireIntensity > 4;
    }

    @Override
    public int getInventorySize() {
        return 27;
    }

    @Override
    public void func_145839_a(NBTTagCompound tag) {
        super.func_145839_a(tag);
        this.fireIntensity = tag.func_74764_b("fireIntensity") ? tag.func_74762_e("fireIntensity") : -1;
        this.facing = tag.func_74764_b("facing") ? EnumFacing.func_82600_a(tag.func_74762_e("facing")) : EnumFacing.UP;
        if (tag.func_74764_b("CookTime"))
            this.cookCounter = tag.func_74762_e("CookTime");
        if (tag.func_74764_b("ContainsValidIngredients"))
            this.containsValidIngredients = tag.func_74767_n("ContainsValidIngredients");
        if (tag.func_74764_b("StokedCooldown"))
            this.stokedCooldownCounter = tag.func_74762_e("StokedCooldown");
        validateInventory();
    }

    @Override
    public NBTTagCompound func_189515_b(NBTTagCompound tag) {
        NBTTagCompound t = super.func_189515_b(tag);
        t.func_74768_a("fireIntensity", this.fireIntensity);
        t.func_74768_a("facing", facing.func_176745_a());
        t.func_74768_a("CookTime", this.cookCounter);
        t.func_74768_a("StokedCooldown", this.stokedCooldownCounter);
        t.func_74757_a("ContainsValidIngredients", this.containsValidIngredients);
        return t;
    }


    @Override
    public void func_73660_a() {
        if (this.getBlockWorld().field_72995_K)
            return;
        if (this.getBlockWorld().func_180495_p(this.field_174879_c).func_177230_c() instanceof BlockCookingPot) {
            IBlockState state = this.getBlockWorld().func_180495_p(this.field_174879_c);
            if (this.fireIntensity != getFireIntensity()) {
                validateFireIntensity();
                this.forceValidation = true;
            }
            if (!isPowered()) {
                this.facing = EnumFacing.UP;
                entityCollision();
                if (this.fireIntensity > 0) {
                    if (this.forceValidation) {
                        validateContents();
                        this.forceValidation = false;
                    }
                    if (this.fireIntensity > 4) {
                        if (this.stokedCooldownCounter < 1)
                            this.cookCounter = 0;
                        this.stokedCooldownCounter = 20;
                        performStokedFireUpdate(getCurrentFireIntensity());
                    } else if (this.stokedCooldownCounter > 0) {
                        this.stokedCooldownCounter -= 1;

                        if (this.stokedCooldownCounter < 1) {
                            this.cookCounter = 0;
                        }
                    } else if (this.stokedCooldownCounter == 0 && this.fireIntensity > 0 && this.fireIntensity < 5)
                        performNormalFireUpdate(getCurrentFireIntensity());
                } else
                    this.cookCounter = 0;
            } else {
                this.cookCounter = 0;
                this.facing = getPoweredSide();
                ejectInventory(DirUtils.rotateFacingAroundY(this.facing, false));
            }

            if (facing != state.func_177229_b(DirUtils.TILTING)) {
                field_145850_b.func_175656_a(field_174879_c, state.func_177226_a(DirUtils.TILTING, facing));
            }


        }
        validateInventory();
        this.scaledCookCounter = this.cookCounter * 1000 / 4350;
    }

    private void entityCollision() {
        if (captureDroppedItems()) {
            getBlockWorld().func_180497_b(field_174879_c, this.func_145838_q(), this.func_145838_q().func_149738_a(getBlockWorld()), 5);
            this.func_70296_d();
        }
    }

    public List<EntityItem> getCaptureItems(World worldIn, BlockPos pos) {
        return worldIn.func_175647_a(EntityItem.class, new AxisAlignedBB(pos.func_177958_n(), pos.func_177956_o(), pos.func_177952_p(), pos.func_177958_n() + 1D, pos.func_177956_o() + 1.5D, pos.func_177952_p() + 1D), EntitySelectors.field_94557_a);
    }

    private boolean captureDroppedItems() {
        boolean insert = false;
        if (!InvUtils.isFull(inventory)) {
            List<EntityItem> items = this.getCaptureItems(getBlockWorld(), func_174877_v());
            for (EntityItem item : items)
                insert &= InvUtils.insertFromWorld(inventory, item, 0, 27, false);
        }
        if (insert) {
            this.getBlockWorld().func_184148_a(null, field_174879_c.func_177958_n(), field_174879_c.func_177956_o(), field_174879_c.func_177952_p(), SoundEvents.field_187638_cR, SoundCategory.PLAYERS, 0.2F, ((getBlockWorld().field_73012_v.nextFloat() - getBlockWorld().field_73012_v.nextFloat()) * 0.7F + 1.0F) * 2.0F);
            if (this.validateInventory()) {
                getBlockWorld().func_180497_b(field_174879_c, this.func_145838_q(), this.func_145838_q().func_149738_a(getBlockWorld()), 5);
            }
            return true;
        }
        return false;
    }

    public void ejectInventory(EnumFacing facing) {
        int index = InvUtils.getFirstOccupiedStackNotOfItem(inventory, Items.field_151118_aC);
        if (index >= 0 && index < inventory.getSlots()) {

            ItemStack stack = inventory.getStackInSlot(index);
            int ejectStackSize = 8;
            if (8 > stack.func_190916_E()) {
                ejectStackSize = stack.func_190916_E();
            }
            BlockPos target = field_174879_c.func_177972_a(facing);
            ItemStack eject = new ItemStack(stack.func_77973_b(), ejectStackSize, stack.func_77952_i());
            InvUtils.copyTags(eject, stack);
            IBlockState targetState = getBlockWorld().func_180495_p(target);
            boolean ejectIntoWorld = getBlockWorld().func_175623_d(target) || targetState.func_177230_c().func_176200_f(getBlockWorld(), target) || !targetState.func_185904_a().func_76220_a() || targetState.func_185900_c(getBlockWorld(), target).field_72337_e < 0.5d;
            if (ejectIntoWorld) {
                this.getBlockWorld().func_184133_a(null, field_174879_c, SoundEvents.field_187638_cR, SoundCategory.BLOCKS, 0.2F, ((getBlockWorld().field_73012_v.nextFloat() - getBlockWorld().field_73012_v.nextFloat()) * 0.7F + 1.0F) * 2.0F);
                ejectStack(getBlockWorld(), target, facing, eject);
                inventory.extractItem(index, ejectStackSize, false);
            }
        }
    }

    public void ejectStack(World world, BlockPos pos, EnumFacing facing, ItemStack stack) {
        Vec3i vec = new BlockPos(0, 0, 0).func_177972_a(facing);
        EntityItem item = new EntityItem(world, pos.func_177958_n() + 0.5F - (vec.func_177958_n() / 4d), pos.func_177956_o() + 0.25D, pos.func_177952_p() + 0.5D - (vec.func_177952_p() / 4d), stack);
        float velocity = 0.05F;
        item.field_70159_w = (double) (vec.func_177958_n() * velocity);
        item.field_70181_x = vec.func_177956_o() * velocity * 0.1;
        item.field_70179_y = (double) (vec.func_177952_p() * velocity);
        item.func_174869_p();
        world.func_72838_d(item);
    }

    @Override
    public void func_70296_d() {
        super.func_70296_d();
        this.forceValidation = true;
        if (this.getBlockWorld() != null) {
            validateInventory();
        }
    }


    public boolean isUseableByPlayer(EntityPlayer player) {
        return this.getBlockWorld().func_175625_s(this.field_174879_c) == this && player.func_70092_e(this.field_174879_c.func_177958_n() + 0.5D, this.field_174879_c.func_177956_o() + 0.5D, this.field_174879_c.func_177952_p() + 0.5D) <= 64.0D;
    }

    public abstract boolean validateStoked();

    public abstract boolean validateUnstoked();

    public void validateContents() {
        this.containsValidIngredients = false;
        if (!isStoked()) {
            containsValidIngredients = validateUnstoked();
        } else {
            containsValidIngredients = validateStoked();
        }
    }

    protected CraftingManagerBulk getCraftingManager(boolean stoked) {
        return stoked ? this.stoked : unstoked;
    }

    public int getCurrentFireIntensity() {
        int fireFactor = 0;

        if (this.fireIntensity > 0) {
            for (int x = -1; x <= 1; x++) {
                for (int z = -1; z <= 1; z++) {
                    int yPos = -1;
                    BlockPos target = field_174879_c.func_177982_a(x, yPos, z);
                    Block block = this.getBlockWorld().func_180495_p(target).func_177230_c();
                    int meta = this.getBlockWorld().func_180495_p(target).func_177230_c().func_180651_a(this.getBlockWorld().func_180495_p(target));
                    if (BWMHeatRegistry.get(block, meta) != null)
                        fireFactor += BWMHeatRegistry.get(block, meta).value;
                }
            }
            if (fireFactor < 5)
                fireFactor = 5;
        }

        return fireFactor;
    }

    private void performNormalFireUpdate(int fireIntensity) {
        if (this.containsValidIngredients) {
            this.cookCounter += fireIntensity;

            if (this.cookCounter >= 4350) {
                attemptToCookNormal();
                this.cookCounter = 0;
            }
        } else
            this.cookCounter = 0;
    }

    private void performStokedFireUpdate(int fireIntensity) {
        if (this.containsValidIngredients) {
            this.cookCounter += fireIntensity;

            if (this.cookCounter >= 4350) {
                attemptToCookStoked();
                this.cookCounter = 0;
            }
        } else
            this.cookCounter = 0;
    }


    private boolean containsItem(Item item) {
        return containsItem(item, OreDictionary.WILDCARD_VALUE);
    }

    private boolean containsItem(Item item, int meta) {
        return InvUtils.getFirstOccupiedStackOfItem(inventory, item) > -1;
    }

    protected boolean attemptToCookNormal() {
        return attemptToCookWithManager(getCraftingManager(false));
    }

    protected boolean attemptToCookStoked() {
        return attemptToCookWithManager(getCraftingManager(true));
    }

    private boolean attemptToCookWithManager(CraftingManagerBulk man) {
        if (man != null) {
            if (man.getCraftingResult(inventory) != null) {
                NonNullList<ItemStack> output = man.craftItem(field_145850_b, this, inventory);
                if (!output.isEmpty()) {
                    for (ItemStack out : output) {
                        if (!out.func_190926_b()) {
                            ItemStack stack = out.func_77946_l();
                            if (!InvUtils.insert(inventory, stack, false).func_190926_b())
                                InvUtils.ejectStackWithOffset(this.getBlockWorld(), this.field_174879_c.func_177984_a(), stack);
                        }
                    }
                    return true;
                }
            }
        }
        return false;
    }

    public int getCookingProgressScaled(int scale) {
        return this.scaledCookCounter * scale / 1000;
    }

    public boolean isCooking() {
        return this.scaledCookCounter > 0;
    }

    private boolean validateInventory() {
        boolean stateChanged = false;
        byte currentSlots = (byte) InvUtils.getOccupiedStacks(inventory);
        if (currentSlots != this.occupiedSlots) {
            this.occupiedSlots = currentSlots;
            stateChanged = true;
        }
        if (getBlockWorld() != null && stateChanged) {
            IBlockState state = getBlockWorld().func_180495_p(field_174879_c);
            getBlockWorld().func_184138_a(field_174879_c, state, state, 3);
        }

        return stateChanged;
    }

    public int getFireIntensity() {
        BlockPos down = field_174879_c.func_177977_b();
        Block block = this.getBlockWorld().func_180495_p(down).func_177230_c();
        int meta = block.func_180651_a(this.getBlockWorld().func_180495_p(down));
        BWMHeatSource source = BWMHeatRegistry.get(block, meta);
        if (source != null)
            return source.value;
        return -1;
    }

    private void validateFireIntensity() {
        BlockPos down = field_174879_c.func_177977_b();
        Block block = this.getBlockWorld().func_180495_p(down).func_177230_c();
        int meta = block.func_180651_a(this.getBlockWorld().func_180495_p(down));
        BWMHeatSource source = BWMHeatRegistry.get(block, meta);
        if (source != null)
            fireIntensity = source.value;
        else
            fireIntensity = -1;
    }

    @Override
    public int getMaxVisibleSlots() {
        return 27;
    }

    @Override
    public int getMechanicalOutput(EnumFacing facing) {
        return 0;
    }

    @Override
    public int getMechanicalInput(EnumFacing facing) {
        return MechanicalUtil.getPowerOutput(field_145850_b, field_174879_c.func_177972_a(facing), facing.func_176734_d());
    }

    @Override
    public int getMaximumInput(EnumFacing facing) {
        return 1;
    }

    @Override
    public int getMinimumInput(EnumFacing facing) {
        return 0;
    }

    @Override
    public World getBlockWorld() {
        return super.func_145831_w();
    }

    @Override
    public BlockPos getBlockPos() {
        return func_174877_v();
    }

    @Override
    public Block getBlock() {
        return func_145838_q();
    }
}
